;********************
;*                  *
;* CONTEXT ROUTINES *
;*                  *
;********************

;*********************************************************
;* Create a new context data block. The executable file  *
;*  will be relocated and a stack block will be created. *
;* Input: a5.l = *_main                                  *
;*        a6.l = *Prosit return area                     *
;*        d5.w = Handle of executable file               *
;*        d3.l = Message Buffer size                     *
;*        d4.l = Stack size                              *
;*        d7.l = ID                                      *
;* Output: d6.w = Handle of context data block           *
;*********************************************************
CreateContextBlock:
	movem.l		a0-a2/a4-a6/d0-d2,-(sp)

	move.l		#CONTEXT_SIZE,-(sp)
	jsr		doorsos::HeapAlloc	;Allocate memory for context block
	addq.l		#4,sp
	move.w		d0,d6			;Store context block handle

	doorsos::DEREF	d0,a4			;*Context block

;	move.l		d7,C_ID(a4)		; Process ID

;Setup the user stack
	move.w		d4,d0
	beq.s		\nostk
	move.l		d4,-(sp)
	jsr		doorsos::HeapAlloc	; Allocate stack
	addq.l		#4,sp
\nostk	move.w		d0,C_STACK_HANDLE(a4)	; Set stack handle
	beq.s		\sdone
	doorsos::DEREF	d0,a2
	lea		0(a2,d4),a2		; We start at the end of the stack
	move.l		#ReturnFromProcess,-(a2);Set return point if process self-terminates
	move.l		a2,C_USP(a4)		; Set USP
\sdone

;Setup the message buffer
	move.w		d3,d0			; Store size
	beq.s		\nombuf
	addq.l		#4,d3			; For size and index
	move.l		d3,-(sp)
	jsr		doorsos::HeapAlloc	; Allocate message buffer
	addq.l		#4,sp
\nombuf	move.w		d0,C_MESSAGEBUFFER_HANDLE(a4)	; Set handle
	beq.s		\mdone
	doorsos::DEREF	d0,a2
	move.w		d4,(a2)+		; Set buffer size
	clr.w		(a2)			; Clear index
\mdone
	move.l		a6,C_PROSIT_DATARETURN(a4)	; Return data point
	clr.w		PD_MESSAGES(a6)			; Clear no. of messages told to program
	move.w		#CS_INIT,C_STATUS_FLAGS(a4)	; Set process' status to RUNNING

;Setup the execution parameters
	move.w		#0,C_SR(a4)		; Set SR
	move.l		a5,C_PC(a4)		; Set PC
	move.w		d5,C_PROG_HANDLE(a4)	; Set program handle

	movem.l		(sp)+,a0-a2/a4-a6/d0-d2
	rts
