;
; IDEA: Forced termination should jump to the _exit point of the program that is to be terminated!
;
; IDEA: A process is only allowed to send one message per execution slot. Can easily be done by
;        jumping to the scheduler after the message has been inserted and the relevant registers
;        have been restored (including return values).
;
; DON'T FORGET to add a6.l as an input to Process_Fork (PROSIT_RETURNDATA!)
;


	include		"calc.h"

	xdef    	_main

	xdef		_exit
	xdef		_comment

	include		"doorsos.h"
	include		"macros.h"
	include		"hexlib.h"

	xdef		prosit@0000	;DRD-routine

	xdef		prosit@0001	;Process_New
	xdef		prosit@0002	;Process_Fork
	xdef		prosit@0003	;Process_Terminate
	xdef		prosit@0004	;Send_Message
	xdef		prosit@0005	;Read_Message

	xdef		prosit@0006	;Sleep
	xdef		prosit@0007	;Wake_Up
	xdef		prosit@0008	;Trigger_Event
	xdef		prosit@0009	;Broadcast_Event

	xdef		prosit@000A	;Get_File_Handle (p_file.asm)
	xdef		prosit@000B	;Execute_File (p_file.asm)

	xdef		prosit@000C	;CurrentProcessID
	xdef		prosit@000D	;Ticks
	xdef		prosit@000E	;ControlKeys
	xdef		prosit@000F	;CurrentKey
	xdef		prosit@0010	;MouseX
	xdef		prosit@0011	;MouseY

	xdef		prosit@0012	;List_Insert
	xdef		prosit@0013	;List_Remove
	xdef		prosit@0014	;List_Previous
	xdef		prosit@0015	;List_Find
	xdef		prosit@0016	;List_Size

	include		"prosit.h"	;Import system call equ:s

C_NEXT			EQU	0
C_ID			EQU	C_NEXT+4
C_D0			EQU	C_ID+2			; D0
C_A0			EQU	C_D0+32			; A0
C_USP			EQU	C_A0+28			; USP
C_SR			EQU	C_USP+4			; SR
C_PC			EQU	C_SR+2			; PC
C_STATUS_FLAGS		EQU	C_PC+4			; System flags

;Process starts of as "running"
CS_SLEEPING		EQU	0
CS_IN_INTERRUPT		EQU	1
CS_INIT			EQU	%0000000000000000
	;Bit 0: Sleeping		Bit 0:
	;Bit 1: In interrupt		Bit 1:
	;Bit 2: 			Bit 2:
	;Bit 3:				Bit 3:
	;Bit 4:				Bit 4:
	;Bit 5:				Bit 5:
	;Bit 6:				Bit 6:
	;Bit 7:				Bit 7:
C_PROG_HANDLE		EQU	C_STATUS_FLAGS+2		; Program handle
C_STACK_HANDLE		EQU	C_PROG_HANDLE+2			; Stack handle
C_PROSIT_DATARETURN	EQU	C_STACK_HANDLE+2		; *Prosit return-data area
C_MESSAGEBUFFER_HANDLE	EQU	C_PROSIT_DATARETURN+4		; Message buffer handle
CONTEXT_SIZE		EQU	C_MESSAGEBUFFER_HANDLE+2




PD_START		EQU	8
PD_MESSAGES		EQU	0
PD_NO_OF_EVENTS		EQU	PD_MESSAGES+2
PD_EVENTS		EQU	PD_NO_OF_EVENTS+2


_main:

;Find and relocate ProBoot process
	pea		folder(pc)		;*Folder name
	pea		program(pc)		;*File name
	bsr		Get_File_Handle		;d5.w = File handle
	addq.l		#8,sp
	move.w		d5,ProBootFileHandle
	beq		_exit			;File didn't exist, quit

	bsr		RelocateFile		;Relocate file

	addq.l		#6,a5			;Skip the DRD section
	move.w		$16(a6),d0		;Offset to export table
	move.w		2(a6,d0.w),d0		;Offset to exported data
	lea		PD_START(a6,d0.w),a6	;a6.l = *Prosit data from file (skipping first word)

;Got all necessary information, create context entry
	move.l		#CONTEXT_SIZE,-(sp)
	jsr		doorsos::HeapAlloc
	addq.l		#4,sp
	move.w		d0,ProBootContextHandle	;Store handle for later use
	doorsos::DEREF	d0,a0			;Get adress of handle

	move.w		ProBootContextHandle,C_ID(a0)
	move.w		d5,C_PROG_HANDLE(a0)
	clr.w		C_STACK_HANDLE(a0)
	clr.w		C_MESSAGEBUFFER_HANDLE(a0)
	move.l		a6,C_PROSIT_DATARETURN(a0)
	move.w		#CS_INIT,C_STATUS_FLAGS(a0)
	clr.w		PD_MESSAGES(a6)

;Setup context table
	clr.l		ContextList
	clr.l		SleepingContextList
	move.b		#0,-(sp)
	move.l		a0,-(sp)
	move.l		#ContextList,-(sp)
	bsr		List_Insert
	add.w		#10,sp

	add.w		#C_D0,a0
	move.l		a0,SMCurrentProcess+4	;Self-modifying code for the scheduler

;Set some variables to their default values
	clr.w		CurrentKey
	move.b		#$FF,ControlKeys
	clr.w		KeyboardDelay
	clr.l		Ticks

;***************************************************************
;* Initialize the scheduler:                                   *
;*  o Point Auto-Int 1 to the Scheduler                        *
;*  o Set Auto-Int 5:s frequency and point it to the KeyReader *
;*  o Point Trap #13 to ToggleProsit                           *
;*  o Point Trap #14 to the Scheduler                          *
;***************************************************************
	move.w		#$0700,d0
	trap		#1
	move.l		INT1_ADDRESS,old_int1
	move.l		INT5_ADDRESS,old_int5
	move.l		TRAP13_ADDRESS,old_trap13
	move.l		TRAP14_ADDRESS,old_trap14
	bclr.b		#2,$600001
	move.l		#Scheduler,INT1_ADDRESS
	move.l		#KeyReader,INT5_ADDRESS
	move.l		#ToggleProsit,TRAP13_ADDRESS
	move.l		#Scheduler,TRAP14_ADDRESS
	bset.b		#2,$600001

	trap		#1
	move.w		#PROSIT_INT5_VALUE,($600016)

;***********************
;* Run ProBoot process *
;***********************
	jsr		(a5)
\wait	;tst.l		ContextList
	;bne.s		\wait			;Wait for all processes to terminate
	tst.w		CurrentKey
	bne.s		\wait			;Wait for the user to release an eventual key

;****************************
;* Deactivate the scheduler *
;****************************
	move.w		#$0700,d0
	trap		#1
	bclr.b		#2,$600001
	move.l		old_int1,INT1_ADDRESS
	move.l		old_int5,INT5_ADDRESS
	move.l		old_trap13,TRAP13_ADDRESS
	move.l		old_trap14,TRAP14_ADDRESS
	bset.b		#2,$600001
	trap		#1
	move.w		#TIOS_INT5_VALUE,($600016)

	move.w		ProBootFileHandle,d0
	move.l		($40),a0
	jsr		(a0)			;Unrelocate ProBoot file

;******************************************************
;* De-allocate ProBoot context block and context list *
;******************************************************
	move.w		ProBootContextHandle,-(sp)
	jsr		doorsos::HeapFree	;De-allocate ProBoot context block
	addq.l		#2,sp

_exit:
	rts





	include		"p_drd.asm"
	include		"p_syscal.asm"
	include		"p_sched.asm"
	include		"p_file.asm"
	include		"p_contxt.asm"
	include		"p_list.asm"

;***********************************************************************


;***********************************************************************

;***********************************************************************

	EVEN
prosit@000C:
CurrentProcessID	dc.w	0
prosit@000D:
Ticks			dc.l	0
prosit@000E:
ControlKeys		dc.w	0
prosit@000F:
CurrentKey		dc.w	0
prosit@0010:
MouseX			dc.w	0
prosit@0011:
MouseY			dc.w	0


folder		dc.b	"main",0
program		dc.b	"proboot",0

_comment
	dc.b	"Prosit v0.58c",0

	BSS

ProBootFileHandle	dc.w	0
ProBootContextHandle	dc.w	0
ContextList		dc.l	0
SleepingContextList	dc.l	0

old_int1		dc.l	0
old_int5		dc.l	0
old_trap13		dc.l	0
old_trap14		dc.l	0

KeyboardDelay		dc.w	0

	END
